package org.xtextcasts.edsl.example

import org.eclipse.core.databinding.DataBindingContext
import org.eclipse.core.databinding.observable.Realm
import org.eclipse.emf.databinding.EMFProperties
import org.eclipse.jface.databinding.swt.SWTObservables
import org.eclipse.jface.databinding.swt.WidgetProperties
import org.eclipse.swt.events.SelectionEvent
import org.eclipse.swt.events.SelectionListener
import org.eclipse.swt.layout.GridData
import org.eclipse.swt.layout.GridLayout
import org.eclipse.swt.widgets.Button
import org.eclipse.swt.widgets.Composite
import org.eclipse.swt.widgets.Display
import org.eclipse.swt.widgets.Group
import org.eclipse.swt.widgets.Label
import org.eclipse.swt.widgets.Shell
import org.eclipse.swt.widgets.Text
import org.xtextcasts.edsl.example.edsl.GroupDef
import org.xtextcasts.edsl.example.edsl.TextInputDef
import org.xtextcasts.edsl.example.forms.MyForm
import org.xtextcasts.edsl.example.model.ModelFactory
import org.xtextcasts.edsl.example.model.ModelPackage
import org.xtextcasts.edsl.example.model.RegistrationForm

import static org.eclipse.swt.SWT.*
import org.eclipse.swt.widgets.Spinner
import org.xtextcasts.edsl.example.edsl.NumberInputDef

class Main {
	Shell shell
	RegistrationForm model
	val bindingContext = new DataBindingContext()

	new(Shell shell, RegistrationForm form) {
		this.shell = shell
		this.model = form
	}

	def static void main(String[] args) {
		val display = new Display();
		val shell = new Shell(display);
		shell.setLayout(new GridLayout(1, false));


		ModelPackage.eINSTANCE.eClass();
		
    	// Retrieve the default factory singleton
    	val factory = ModelFactory.eINSTANCE;
	    val form = factory.createRegistrationForm();

		Realm.runWithDefault(SWTObservables.getRealm(display), [|
			new Main(shell, form).constructForm(new MyForm());
			shell.pack();
		 	shell.open();
			while (!shell.isDisposed()) {
				if (!display.readAndDispatch())
					display.sleep();
			}
			display.dispose();
		])
	}

	def void constructForm(MyForm myForm) {
		myForm.specifyForm
		shell.text = myForm.title ?: ""
		shell.layout = new GridLayout(1, false)
		
		for (elementDef : myForm.form.elementList) {
			constructElement(elementDef, shell)
		}
		
		new Composite(shell, NONE) => [
			layout = new GridLayout(2, false)
			layoutData = new GridData(RIGHT, BOTTOM, true, true)
			new Button(it, NONE) => [
				text = "OK"
				onSelect [
					println(model)
					System::exit(0)
				]
			]
			new Button(it, NONE) => [
				text = "Cancel"
				onSelect [
					System::exit(0)
				]
			]
		]
	}
	
	def dispatch void constructElement(GroupDef compositeDef, Composite parent) {
		val composite = new Group(parent, BORDER)
		composite.text = compositeDef.text
		composite.layout = new GridLayout(2, false)
		composite.layoutData = new GridData(FILL, FILL, true, false)
		for (elementDef : compositeDef.elementList) {
			constructElement(elementDef, composite)
		}
	}

	def dispatch void constructElement(TextInputDef textInputDef, Composite parent) {
		val label = new Label(parent, NONE);
		label.setText(textInputDef.label);
		val text = new Text(parent, BORDER);
		text.layoutData = new GridData(FILL, CENTER, true, false) => [
			widthHint = 150
		]
		
	    bindingContext.bindValue(
	    	WidgetProperties::text(Modify).observe(text),
			EMFProperties::value(textInputDef.attribute).observe(model)	    	
	    );
		
	}
	def dispatch void constructElement(NumberInputDef numberInputDef, Composite parent) {
		val label = new Label(parent, NONE);
		label.setText(numberInputDef.label);
		val spinner = new Spinner(parent, BORDER);
		spinner.layoutData = new GridData(FILL, CENTER, true, false) => [
			widthHint = 150
		]
		if (numberInputDef.maximum != null) {
			spinner.maximum = numberInputDef.maximum
		}
		if (numberInputDef.minimum != null) {
			spinner.minimum = numberInputDef.minimum
		}
		
	    bindingContext.bindValue(
	    	WidgetProperties::selection.observe(spinner),
			EMFProperties::value(numberInputDef.attribute).observe(model)	    	
	    );
		
	}

	def static onSelect(Button button, (SelectionEvent) => void proc) {
		button.addSelectionListener(new OnSelect(proc))
	}
}

@Data
class OnSelect implements SelectionListener {
	(SelectionEvent)=>void proc

	override void widgetDefaultSelected(SelectionEvent e) {
	}

	override void widgetSelected(SelectionEvent e) {
		proc.apply(e)
	}
}
